@@ -36,6 +36,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <stdlib.h>
+#include <limits.h>
 #include <mntent.h>
 #include <sys/ioctl.h>		/* for _IO */
 #include <sys/utsname.h>
@@ -51,18 +52,24 @@
 #include "xstrncpy.h"
 #include "nls.h"
 #include "blkdev.h"
+#include "pathnames.h"
+#include "pttype.h"
+#include "wholedisk.h"
 
 #ifdef HAVE_LIBUUID
-#include <uuid/uuid.h>
+# ifdef HAVE_UUID_UUID_H
+#  include <uuid/uuid.h>
+#else
+# include <uuid.h>
+# endif
 #endif
 
 static char * program_name = "mkswap";
 static char * device_name = NULL;
 static int DEV = -1;
-static unsigned long PAGES = 0;
+static unsigned long long PAGES = 0;
 static unsigned long badpages = 0;
 static int check = 0;
-static int version = -1;
 
 #define SELINUX_SWAPFILE_TYPE	"swapfile_t"
 
@@ -131,10 +138,6 @@ is_sparc64(void) {
  *
  * What to do? Let us allow the user to specify the pagesize explicitly.
  *
- * Update 05-Feb-2007 (kzak):
- *      - use sysconf(_SC_PAGESIZE) to be consistent with the rest of
- *        util-linux code.  It is the standardized and preferred way of
- *        querying page size.
  */
 static int user_pagesize;
 static int pagesize;
@@ -144,7 +147,7 @@ struct swap_header_v1 *p;
 static void
 init_signature_page(void) {
 
-	int kernel_pagesize = pagesize = (int) sysconf(_SC_PAGESIZE);
+	int kernel_pagesize = pagesize = getpagesize();
 
 	if (user_pagesize) {
 		if ((user_pagesize & (user_pagesize-1)) ||
@@ -215,7 +218,6 @@ write_uuid_and_label(unsigned char *uuid, char *volume_name) {
  * swap space as it can handle, but until 2.3.4 it would return an error
  * to swapon() if the swapspace was too large.
  */
-#define V0_MAX_PAGES	(8 * (pagesize - 10))
 /* Before 2.2.0pre9 */
 #define V1_OLD_MAX_PAGES	((0x7fffffff / pagesize) - 1)
 /* Since 2.2.0pre9, before 2.3.4:
@@ -266,70 +268,12 @@ It is roughly 2GB on i386, PPC, m68k, ARM, 1GB on sparc, 512MB on mips,
 */
 
 #define MAX_BADPAGES	((pagesize-1024-128*sizeof(int)-10)/sizeof(int))
-
-/*
- * One more point of lossage - Linux swapspace really is a mess.
- * The definition of the bitmap used is architecture dependent,
- * and requires one to know whether the machine is bigendian,
- * and if so, whether it will use 32-bit or 64-bit units in
- * test_bit().
- * davem writes: "... is based upon an unsigned long type of
- * the cpu and the native endianness".
- * So, it seems we can write `unsigned long' below.
- * However, sparc64 uses 64-bit units in the kernel, while
- * mkswap may have been translated with 32-bit longs. Thus,
- * we need an explicit test for version 0 swap on sparc64.
- */
-
-static void
-bit_set (unsigned long *addr, unsigned int nr) {
-	unsigned int r, m;
-
-	if(is_be64()) {
-		unsigned long long *bitmap = (unsigned long long *) addr;
-		unsigned long long bitnum = (unsigned long long) nr;
-		unsigned long long rl, ml;
-
-		bitmap += bitnum / (8 * sizeof(long long));
-		rl = *bitmap;
-		ml = 1ULL << (bitnum & (8ULL * sizeof(long long) - 1ULL));
-		*bitmap = rl | ml;
-		return;
-	}
-
-	addr += nr / (8 * sizeof(unsigned long));
-	r = *addr;
-	m = 1 << (nr & (8 * sizeof(unsigned long) - 1));
-	*addr = r | m;
-}
-
-static int
-bit_test_and_clear (unsigned long *addr, unsigned int nr) {
-	unsigned int r, m;
-
-	if(is_be64()) {
-		unsigned long long *bitmap = (unsigned long long *) addr;
-		unsigned long long bitnum = (unsigned long long) nr;
-		unsigned long long rl, ml;
-
-		bitmap += bitnum / (8 * sizeof(long long));
-		rl = *bitmap;
-		ml = 1ULL << (bitnum & (8ULL * sizeof(long long) - 1ULL));
-		*bitmap = rl & ~ml;
-		return ((rl & ml) != 0ULL);
-	}
-
-	addr += nr / (8 * sizeof(unsigned long));
-	r = *addr;
-	m = 1 << (nr & (8 * sizeof(unsigned long) - 1));
-	*addr = r & ~m;
-	return (r & m) != 0;
-}
+#define MIN_GOODPAGES	10
 
 static void
 usage(void) {
 	fprintf(stderr,
-		_("Usage: %s [-c] [-v0|-v1] [-pPAGESZ] [-L label] [-U UUID] /dev/name [blocks]\n"),
+		_("Usage: %s [-c] [-pPAGESZ] [-L label] [-U UUID] /dev/name [blocks]\n"),
 		program_name);
 	exit(1);
 }
@@ -340,21 +284,11 @@ die(const char *str) {
 	exit(1);
 }
 
-static void
-page_ok(int page) {
-	if (version==0)
-		bit_set(signature_page, page);
-}
-
 static void
 page_bad(int page) {
-	if (version == 0)
-		bit_test_and_clear(signature_page, page);
-	else {
-		if (badpages == MAX_BADPAGES)
-			die(_("too many bad pages"));
-		p->badpages[badpages] = page;
-	}
+	if (badpages == MAX_BADPAGES)
+		die(_("too many bad pages"));
+	p->badpages[badpages] = page;
 	badpages++;
 }
 
@@ -369,10 +303,8 @@ check_blocks(void) {
 		die(_("Out of memory"));
 	current_page = 0;
 	while (current_page < PAGES) {
-		if (!check) {
-			page_ok(current_page++);
+		if (!check)
 			continue;
-		}
 		if (do_seek && lseek(DEV,current_page*pagesize,SEEK_SET) !=
 		    current_page*pagesize)
 			die(_("seek failed in check_blocks"));
@@ -380,7 +312,6 @@ check_blocks(void) {
 			page_bad(current_page++);
 			continue;
 		}
-		page_ok(current_page++);
 	}
 	if (badpages == 1)
 		printf(_("one bad page\n"));
@@ -388,37 +319,8 @@ check_blocks(void) {
 		printf(_("%lu bad pages\n"), badpages);
 }
 
-static long
-valid_offset (int fd, off_t offset) {
-	char ch;
-
-	if (lseek (fd, offset, 0) < 0)
-		return 0;
-	if (read (fd, &ch, 1) < 1)
-		return 0;
-	return 1;
-}
-
-static off_t
-find_size (int fd) {
-	off_t high, low;
-
-	low = 0;
-	for (high = 1; high > 0 && valid_offset (fd, high); high *= 2)
-		low = high;
-	while (low < high - 1) {
-		const off_t mid = (low + high) / 2;
-
-		if (valid_offset (fd, mid))
-			low = mid;
-		else
-			high = mid;
-	}
-	return (low + 1);
-}
-
-/* return size in pages, to avoid integer overflow */
-static unsigned long
+/* return size in pages */
+static unsigned long long
 get_size(const char  *file) {
 	int	fd;
 	unsigned long long size;
@@ -431,7 +333,7 @@ get_size(const char  *file) {
 	if (blkdev_get_size(fd, &size) == 0)
 		size /= pagesize;
 	else
-		size = find_size(fd) / pagesize;
+		size = blkdev_find_size(fd) / pagesize;
 
 	close(fd);
 	return size;
@@ -454,7 +356,7 @@ check_mount(void) {
 	FILE * f;
 	struct mntent * mnt;
 
-	if ((f = setmntent (MOUNTED, "r")) == NULL)
+	if ((f = setmntent (_PATH_MOUNTED, "r")) == NULL)
 		return 0;
 	while ((mnt = getmntent (f)) != NULL)
 		if (strcmp (device_name, mnt->mnt_fsname) == 0)
@@ -483,15 +385,57 @@ write_all(int fd, const void *buf, size_t count) {
 	return 0;
 }
 
+static void
+zap_bootbits(int fd, const char *devname, int force)
+{
+	const char *type = NULL;
+	int zap = 1;
+
+	if (!force) {
+		if (lseek(fd, 0, SEEK_SET) != 0)
+	                die(_("unable to rewind swap-device"));
+
+		if (is_whole_disk_fd(fd, devname))
+			/* don't zap bootbits on whole disk -- we know nothing
+			 * about bootloaders on the device */
+			zap = 0;
+
+		else if ((type = get_pt_type_fd(fd)))
+			/* don't zap partition table */
+			zap = 0;
+	}
+
+	if (zap) {
+		char buf[1024];
+
+		if (lseek(fd, 0, SEEK_SET) != 0)
+	                die(_("unable to rewind swap-device"));
+
+		memset(buf, 0, sizeof(buf));
+		if (write_all(fd, buf, sizeof(buf)))
+			die(_("unable to erase bootbits sectors"));
+		return;
+	}
+
+	fprintf(stderr, _("%s: %s: warning: don't erase bootbits sectors\n"),
+		program_name, devname);
+	if (type)
+		fprintf(stderr, _("        (%s partition table detected). "), type);
+	else
+		fprintf(stderr, _("        on whole disk. "));
+	fprintf(stderr, "Use -f to force.\n");
+}
+
 int
 main(int argc, char ** argv) {
 	struct stat statbuf;
 	int i;
-	unsigned long maxpages;
-	unsigned long goodpages;
-	unsigned long sz;
+	unsigned long long maxpages;
+	unsigned long long goodpages;
+	unsigned long long sz;
 	off_t offset;
 	int force = 0;
+	int version = 1;
 	char *block_count = 0;
 	char *pp;
 	char *opt_label = NULL;
@@ -563,6 +507,12 @@ main(int argc, char ** argv) {
 			usage();
 	}
 
+	if (version != 1) {
+		fprintf(stderr, _("%s: does not support swapspace version %d.\n"),
+			program_name, version);
+		exit(EXIT_FAILURE);
+	}
+
 #ifdef HAVE_LIBUUID
 	if(opt_uuid) {
 		if (uuid_parse(opt_uuid, uuid_dat) != 0)
@@ -585,7 +535,7 @@ main(int argc, char ** argv) {
 		   explicitly */
 		char *tmp;
 		int blocks_per_page = pagesize/1024;
-		PAGES = strtoul(block_count,&tmp,0)/blocks_per_page;
+		PAGES = strtoull(block_count,&tmp,0)/blocks_per_page;
 		if (*tmp)
 			usage();
 	}
@@ -595,61 +545,35 @@ main(int argc, char ** argv) {
 	} else if (PAGES > sz && !force) {
 		fprintf(stderr,
 			_("%s: error: "
-			  "size %lu KiB is larger than device size %lu KiB\n"),
+			  "size %llu KiB is larger than device size %llu KiB\n"),
 			program_name,
 			PAGES*(pagesize/1024), sz*(pagesize/1024));
 		exit(1);
 	}
 
-	if (version == -1) {
-		/* labels only for v1 */
-		if (opt_label)
-			version = 1;
-		else
-		/* use version 1 as default, if possible */
-		if (get_linux_version() < KERNEL_VERSION(2,1,117))
-			version = 0;
-		else if (pagesize < 2048)
-			version = 0;
-		else
-			version = 1;
-	}
-	if (version != 0 && version != 1) {
-		fprintf(stderr, _("%s: error: unknown version %d\n"),
-			program_name, version);
-		usage();
-	}
-
-	if (PAGES < 10) {
+	if (PAGES < MIN_GOODPAGES) {
 		fprintf(stderr,
 			_("%s: error: swap area needs to be at least %ld KiB\n"),
-			program_name, (long)(10 * pagesize/1024));
+			program_name, (long)(MIN_GOODPAGES * pagesize/1024));
 		usage();
 	}
 
-	if (version == 0)
-		maxpages = V0_MAX_PAGES;
-	else if (get_linux_version() >= KERNEL_VERSION(2,3,4))
-		maxpages = PAGES;
+#ifdef __linux__
+	if (get_linux_version() >= KERNEL_VERSION(2,3,4))
+		maxpages = UINT_MAX + 1ULL;
 	else if (get_linux_version() >= KERNEL_VERSION(2,2,1))
 		maxpages = V1_MAX_PAGES;
 	else
+#endif
 		maxpages = V1_OLD_MAX_PAGES;
 
 	if (PAGES > maxpages) {
 		PAGES = maxpages;
 		fprintf(stderr,
-			_("%s: warning: truncating swap area to %ld KiB\n"),
+			_("%s: warning: truncating swap area to %llu KiB\n"),
 			program_name, PAGES * pagesize / 1024);
 	}
 
-	if (opt_label && version == 0) {
-		fprintf(stderr,
-			_("%s: error: label only with v1 swap area\n"),
-			program_name);
-		usage();
-	}
-
 	if (stat(device_name, &statbuf) < 0) {
 		perror(device_name);
 		exit(EXIT_FAILURE);
@@ -681,52 +605,26 @@ main(int argc, char ** argv) {
 		exit(1);
 	}
 
-#ifdef __sparc__
-	if (!force && version == 0) {
-		/* Don't overwrite partition table unless forced */
-		unsigned char *buffer = (unsigned char *)signature_page;
-		unsigned short *q, sum;
-
-		if (read(DEV, buffer, 512) != 512)
-			die(_("fatal: first page unreadable"));
-		if (buffer[508] == 0xDA && buffer[509] == 0xBE) {
-			q = (unsigned short *)(buffer + 510);
-			for (sum = 0; q >= (unsigned short *) buffer;)
-				sum ^= *q--;
-			if (!sum) {
-				fprintf(stderr, _("\
-%s: Device '%s' contains a valid Sun disklabel.\n\
-This probably means creating v0 swap would destroy your partition table.\n\
-No swap was created. If you really want to create v0 swap on that device,\n\
-use the -f option to force it.\n"),
-					program_name, device_name);
-				exit(1);
-			}
-		}
-	}
-#endif
-
-	if (version == 0 || check)
+	if (check)
 		check_blocks();
-	if (version == 0 && !bit_test_and_clear(signature_page,0))
-		die(_("fatal: first page unreadable"));
-	if (version == 1) {
-		p->version = version;
-		p->last_page = PAGES-1;
-		p->nr_badpages = badpages;
-	}
+
+	zap_bootbits(DEV, device_name, force);
+
+	p->version = 1;
+	p->last_page = PAGES-1;
+	p->nr_badpages = badpages;
+
+	if (badpages > PAGES - MIN_GOODPAGES)
+		die(_("Unable to set up swap-space: unreadable"));
 
 	goodpages = PAGES - badpages - 1;
-	if ((long) goodpages <= 0)
-		die(_("Unable to set up swap-space: unreadable"));
-	printf(_("Setting up swapspace version %d, size = %llu KiB\n"),
-		version, (unsigned long long)goodpages * pagesize / 1024);
-	write_signature((version == 0) ? "SWAP-SPACE" : "SWAPSPACE2");
+	printf(_("Setting up swapspace version 1, size = %llu KiB\n"),
+		goodpages * pagesize / 1024);
 
-	if (version == 1)
-		write_uuid_and_label(uuid, opt_label);
+	write_signature("SWAPSPACE2");
+	write_uuid_and_label(uuid, opt_label);
 
-	offset = ((version == 0) ? 0 : 1024);
+	offset = 1024;
 	if (lseek(DEV, offset, SEEK_SET) != offset)
 		die(_("unable to rewind swap-device"));
 	if (write_all(DEV, (char *) signature_page + offset,
