@@ -102,23 +102,20 @@
 #include <utmp.h>
 #include <setjmp.h>
 #include <stdlib.h>
-#include <string.h>
 #include <sys/syslog.h>
 #include <sys/sysmacros.h>
+#include <linux/major.h>
 #include <netdb.h>
 #ifdef HAVE_LIBAUDIT
 # include <libaudit.h>
 #endif
+
 #include "pathnames.h"
 #include "my_crypt.h"
 #include "login.h"
 #include "xstrncpy.h"
 #include "nls.h"
 
-#include <sys/sysmacros.h>
-#include <linux/major.h>
-
-#include <utmp.h>
 
 #ifdef HAVE_SECURITY_PAM_MISC_H
 #  include <security/pam_appl.h>
@@ -342,6 +339,34 @@ logaudit(const char *tty, const char *username, const char *hostname,
 # define logaudit(tty, username, hostname, pwd, status)
 #endif /* HAVE_LIBAUDIT */
 
+#ifdef HAVE_SECURITY_PAM_MISC_H
+/* encapsulate stupid "void **" pam_get_item() API */
+int
+get_pam_username(pam_handle_t *pamh, char **name)
+{
+	const void *item = (void *) *name;
+	int rc;
+	rc = pam_get_item(pamh, PAM_USER, &item);
+	*name = (char *) item;
+	return rc;
+}
+#endif
+
+/*
+ * We need to check effective UID/GID. For example $HOME could be on root
+ * squashed NFS or on NFS with UID mapping and access(2) uses real UID/GID.
+ * The open(2) seems as the surest solution.
+ * -- kzak@redhat.com (10-Apr-2009)
+ */
+int
+effective_access(const char *path, int mode)
+{
+	int fd = open(path, mode);
+	if (fd != -1)
+		close(fd);
+	return fd == -1 ? -1 : 0;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -353,7 +378,7 @@ main(int argc, char **argv)
     int ask, fflag, hflag, pflag, cnt, errsv;
     int quietlog, passwd_req;
     char *domain, *ttyn;
-    char tbuf[MAXPATHLEN + 2], tname[sizeof(_PATH_TTY) + 10];
+    char tbuf[MAXPATHLEN + 2], tname[sizeof(_PATH_DEV_TTY) + 10];
     char *termenv;
     char *childArgv[10];
     char *buff;
@@ -474,7 +499,7 @@ main(int argc, char **argv)
 
     if (ttyn == NULL || *ttyn == '\0') {
 	/* no snprintf required - see definition of tname */
-	sprintf(tname, "%s??", _PATH_TTY);
+	snprintf(tname, sizeof(tname), "%s??", _PATH_DEV_TTY);
 	ttyn = tname;
     }
 
@@ -542,10 +567,10 @@ main(int argc, char **argv)
      * Therefore, we are safe not setting it to anything
      */
 
-    retcode = pam_start("login",username, &conv, &pamh);
+    retcode = pam_start(hflag?"remote":"login",username, &conv, &pamh);
     if(retcode != PAM_SUCCESS) {
-	fprintf(stderr, _("login: PAM Failure, aborting: %s\n"),
-		pam_strerror(pamh, retcode));
+	fprintf(stderr, _("%s: PAM failure, aborting: %s\n"),
+		"login", pam_strerror(pamh, retcode));
 	syslog(LOG_ERR, _("Couldn't initialize PAM: %s"),
 	       pam_strerror(pamh, retcode));
 	exit(99);
@@ -575,6 +600,12 @@ main(int argc, char **argv)
     fprintf(stderr,"\033(K");
 #endif
 
+    if (username) {
+	/* we need't the original username. We have to follow PAM. */
+	free(username);
+	username = NULL;
+    }
+
     /* if fflag == 1, then the user has already been authenticated */
     if (fflag && (getuid() == 0))
 	passwd_req = 0;
@@ -585,9 +616,7 @@ main(int argc, char **argv)
 	int failcount=0;
 
 	/* if we didn't get a user on the command line, set it to NULL */
-	pam_get_item(pamh,  PAM_USER, (const void **) &username);
-	if (!username)
-		pam_set_item(pamh, PAM_USER, NULL);
+	get_pam_username(pamh, &username);
 
 	/* there may be better ways to deal with some of these
 	   conditions, but at least this way I don't think we'll
@@ -601,7 +630,7 @@ main(int argc, char **argv)
 	       (retcode == PAM_USER_UNKNOWN) ||
 	       (retcode == PAM_CRED_INSUFFICIENT) ||
 	       (retcode == PAM_AUTHINFO_UNAVAIL))) {
-	    pam_get_item(pamh, PAM_USER, (const void **) &username);
+	    get_pam_username(pamh, &username);
 
 	    syslog(LOG_NOTICE,_("FAILED LOGIN %d FROM %s FOR %s, %s"),
 		   failcount, hostname, username, pam_strerror(pamh, retcode));
@@ -614,7 +643,7 @@ main(int argc, char **argv)
 	}
 
 	if (retcode != PAM_SUCCESS) {
-	    pam_get_item(pamh, PAM_USER, (const void **) &username);
+	    get_pam_username(pamh, &username);
 
 	    if (retcode == PAM_MAXTRIES)
 		syslog(LOG_NOTICE,_("TOO MANY LOGIN TRIES (%d) FROM %s FOR "
@@ -650,7 +679,7 @@ main(int argc, char **argv)
      * Grab the user information out of the password file for future usage
      * First get the username that we are actually using, though.
      */
-    retcode = pam_get_item(pamh, PAM_USER, (const void **) &username);
+    retcode = get_pam_username(pamh, &username);
     PAM_FAIL_CHECK;
 
     if (!username || !*username) {
@@ -871,7 +900,7 @@ main(int argc, char **argv)
 		sprintf(tmpstr, "%s/%s", pwd->pw_dir, _PATH_HUSHLOGIN);
 		setregid(-1, pwd->pw_gid);
 		setreuid(0, pwd->pw_uid);
-		quietlog = (access(tmpstr, R_OK) == 0);
+		quietlog = (effective_access(tmpstr, O_RDONLY) == 0);
 		setuid(0); /* setreuid doesn't do it alone! */
 		setreuid(ruid, 0);
 		setregid(-1, egid);
@@ -1304,12 +1333,12 @@ getloginname(void) {
 
 static void
 timedout2(int sig) {
-	struct termio ti;
+	struct termios ti;
 
 	/* reset echo */
-	ioctl(0, TCGETA, &ti);
+	tcgetattr(0, &ti);
 	ti.c_lflag |= ECHO;
-	ioctl(0, TCSETA, &ti);
+	tcsetattr(0, TCSANOW, &ti);
 	exit(0);			/* %% */
 }
 
@@ -1331,7 +1360,7 @@ rootterm(char * ttyn)
     char buf[100],*p;
     int cnt, more = 0;
 
-    fd = open(SECURETTY, O_RDONLY);
+    fd = open(_PATH_SECURETTY, O_RDONLY);
     if(fd < 0) return 1;
 
     /* read each line in /etc/securetty, if a line matches our ttyline
@@ -1418,7 +1447,13 @@ dolastlog(int quiet) {
 	    lseek(fd, (off_t)pwd->pw_uid * sizeof(ll), SEEK_SET);
 	}
 	memset((char *)&ll, 0, sizeof(ll));
-	time(&ll.ll_time);
+
+	{
+		time_t t;
+		time(&t);
+		ll.ll_time = t; /* ll_time is always 32bit */
+	}
+
 	xstrncpy(ll.ll_line, tty_name, sizeof(ll.ll_line));
 	if (hostname)
 	    xstrncpy(ll.ll_host, hostname, sizeof(ll.ll_host));
