@@ -19,7 +19,6 @@
 #include "fsprobe.h"
 #include "pathnames.h"
 #include "nls.h"
-#include "realpath.h"
 
 #define streq(s, t)	(strcmp ((s), (t)) == 0)
 
@@ -326,46 +325,39 @@ has_uuid(const char *device, const char *uuid){
 	return ret;
 }
 
-/* Find the entry (SPEC,DIR) in fstab -- spec and dir must be canonicalized! */
+/* Find the entry (DEV,DIR) in fstab -- spec and dir must be canonicalized! */
 struct mntentchn *
-getfs_by_specdir (const char *spec, const char *dir) {
+getfs_by_devdir (const char *dev, const char *dir) {
 	struct mntentchn *mc, *mc0;
 
 	mc0 = fstab_head();
 
 	for (mc = mc0->nxt; mc && mc != mc0; mc = mc->nxt) {
+		int ok = 1;
+
 		/* dir */
 		if (!streq(mc->m.mnt_dir, dir)) {
 			char *dr = canonicalize(mc->m.mnt_dir);
-			int ok = 0;
-
-			if (streq(dr, dir))
-				ok = 1;
-			free(dr);
-			if (!ok)
-				continue;
+			ok = streq(dr, dir);
+			my_free(dr);
 		}
 
 		/* spec */
-		if (!streq(mc->m.mnt_fsname, spec)) {
-			char *fs = canonicalize_spec(mc->m.mnt_fsname);
-			int ok = 0;
+		if (ok && !streq(mc->m.mnt_fsname, dev)) {
+			const char *fs = mc->m.mnt_fsname;
 
-			if (streq(fs, spec))
-				ok = 1;
-			else if (strncmp (fs, "LABEL=", 6) == 0) {
-				if (has_label(spec, fs + 6))
-					ok = 1;
+			if (strncmp (fs, "LABEL=", 6) == 0) {
+				ok = has_label(dev, fs + 6);
+			} else if (strncmp (fs, "UUID=", 5) == 0) {
+				ok = has_uuid(dev, fs + 5);
+			} else {
+				fs = canonicalize_spec(mc->m.mnt_fsname);
+				ok = streq(fs, dev);
+				my_free(fs);
 			}
-			else if (strncmp (fs, "UUID=", 5) == 0) {
-				if (has_uuid(spec, fs + 5))
-					ok = 1;
-			}
-			free(fs);
-			if (!ok)
-				continue;
 		}
-		return mc;
+		if (ok)
+			return mc;
 	}
 
 	return NULL;
@@ -396,13 +388,13 @@ getfs_by_dir (const char *dir) {
 /* Find the device SPEC in fstab.  */
 struct mntentchn *
 getfs_by_spec (const char *spec) {
-	char *name, *value, *cspec;
+	char *name = NULL, *value = NULL, *cspec;
 	struct mntentchn *mc = NULL;
 
 	if (!spec)
 		return NULL;
 
-	if (parse_spec(spec, &name, &value) != 0)
+	if (fsprobe_parse_spec(spec, &name, &value) != 0)
 		return NULL;				/* parse error */
 
 	if (name) {
@@ -412,6 +404,7 @@ getfs_by_spec (const char *spec) {
 			mc = getfs_by_uuid (value);
 
 		free(name);
+		free(value);
 		return mc;
 	}
 
@@ -924,17 +917,14 @@ update_mtab (const char *dir, struct my_mntent *instead) {
  *  number and writes the number back to the file.
  */
 /* dummy */
-const char *fsprobe_get_label_by_devname(const char *spec) { return NULL; }
-const char *fsprobe_get_uuid_by_devname(const char *spec) { return NULL; }
+char *fsprobe_get_label_by_devname(const char *spec) { return NULL; }
+char *fsprobe_get_uuid_by_devname(const char *spec) { return NULL; }
+int fsprobe_parse_spec(const char *spec, char **name, char **value) { return 0; }
 struct my_mntent *my_getmntent (mntFILE *mfp) { return NULL; }
 mntFILE *my_setmntent (const char *file, char *mode) { return NULL; }
 void my_endmntent (mntFILE *mfp) { }
 int my_addmntent (mntFILE *mfp, struct my_mntent *mnt) { return 0; }
 
-char *canonicalize (const char *path) {  return NULL; }
-char *canonicalize_spec (const char *path) { return NULL; }
-int is_pseudo_fs(const char *type) { return 0; };
-
 int
 main(int argc, char **argv)
 {
